@@ -261,8 +261,9 @@ class Cell(object):
     def _get_rotation(self):
         '''Check if there exists a corresponding file
         with rotation angles'''
-        if os.path.isfile(self.morphology[0:-4]+'.rot'):
-            rotation_file = self.morphology[0:-4]+'.rot'
+        base = os.path.splitext(self.morphology)[0]
+        if os.path.isfile(base+'.rot'):
+            rotation_file = base+'.rot'
             rotation_data = open(rotation_file)
             rotation = {}
             for line in rotation_data:
@@ -420,7 +421,10 @@ class Cell(object):
                     command = cmd1 + syntype + cmd2  
                     exec(command)
                     for param in kwargs.keys():
-                        exec('syn.' + param + '=' + str(kwargs[param]))
+                        try:
+                            exec('syn.' + param + '=' + str(kwargs[param]))
+                        except:
+                            pass
                     self.synlist.append(syn)  
 
                     #create NetCon
@@ -634,7 +638,8 @@ class Cell(object):
     
     def simulate(self, electrode=None, rec_imem=False, rec_vmem=False,
                  rec_ipas=False, rec_icap=False,
-                 rec_isyn=False, rec_vmemsyn=False, rec_istim=False):
+                 rec_isyn=False, rec_vmemsyn=False, rec_istim=False,
+                 rec_variables=[]):
         '''
         This is the main function running the simulation of the NEURON model.
         Start NEURON simulation and record variables specified by arguments.
@@ -654,9 +659,10 @@ class Cell(object):
             rec_isyn:   record synaptic currents of from Synapse class instances
             rec_vmemsyn:    record membrane voltage of segments with Synapse
             rec_istim:  record currents of StimIntraElectrode
+            rec_variables: list of variables to record, i.e arg=['cai', ]
         '''
         self._set_soma_volt_recorder()
-        self._set_time_recorder()
+        self._collect_tvec()
         
         if rec_imem:
             self._set_imem_recorders()
@@ -666,6 +672,8 @@ class Cell(object):
             self._set_ipas_recorders()
         if rec_icap:
             self._set_icap_recorders()
+        if len(rec_variables) > 0:
+            self._set_variable_recorders(rec_variables)
         
         #run fadvance until t >= tstopms, and calculate LFP if asked for
         if electrode == None:
@@ -678,54 +686,33 @@ class Cell(object):
                 raise ValueError, 'timeres_NEURON != timeres_python'
             _run_simulation_with_electrode(self, electrode)
         
-        self._collect_tvec()
-        
+        #somatic trace
         self.somav = np.array(self.somav)
         
         if rec_imem:
-            self._calc_imem()
-        
+            self._calc_imem()        
         if rec_ipas:
-            self._calc_ipas()
-        
+            self._calc_ipas()        
         if rec_icap:
-            self._calc_icap()
-        
+            self._calc_icap()        
         if rec_vmem:
-            self._collect_vmem()
-        
+            self._collect_vmem()        
         if rec_isyn:
-            self._collect_isyn()
-        
+            self._collect_isyn()        
         if rec_vmemsyn:
-            self._collect_vsyn()
-        
+            self._collect_vsyn()        
         if rec_istim:
             self._collect_istim()
+        if len(rec_variables) > 0:
+            self._collect_rec_variables(rec_variables)
 
     def _collect_tvec(self):
         '''
         Set the tvec to be a monotonically increasing numpy array after sim.
         '''
-        #fixing tvec, need to be monotonically increasing, from 0-tstopms
-        self.tvec = np.array(self.tvec)
-        if self.tstartms == 0:
-            self.tvec[1:] = self.tvec[1:] + self.timeres_NEURON
-        elif self.tstartms < 0:
-            if self.tvec[0] > -self.timeres_python and \
-                    self.tvec[0] < self.timeres_python:
-                if self.tvec[0] <= -self.timeres_NEURON+self.timeres_NEURON/10.:
-                    self.tvec += self.timeres_NEURON
-                else:
-                    pass
-            else:
-                self.tvec += self.timeres_NEURON
-        else:
-            #if self.verbose:
-            print 'adding %.3f to tvec' % self.timeres_NEURON
-            self.tvec[1:] = self.tvec[1:] + self.timeres_NEURON
+        self.tvec = np.arange(self.tstopms / self.timeres_python + 1) \
+                            * self.timeres_python
         
-
     def _calc_imem(self):
         '''
         Fetch the vectors from the memireclist and calculate self.imem
@@ -773,7 +760,7 @@ class Cell(object):
             if syn.record_current:
                 syn.collect_current(self)
             else:
-                raise Exception, 'must set record_current=True for synapse'
+                raise Exception, 'must set record_current=True in Synapse class'
         self.synireclist = None
         del self.synireclist
     
@@ -797,7 +784,21 @@ class Cell(object):
                 raise Exception, 'must set record_current=True for pointp.'
         self.stimireclist = None
         del self.stimireclist
-       
+        
+    def _collect_rec_variables(self, rec_variables):
+        '''
+        Create dict of np.arrays from recorded variables, each dictionary
+        element named as the corresponding recorded variable name, i.e 'cai'
+        '''
+        self.rec_variables = {}
+        i = 0
+        for values in self.recvariablesreclist:
+            self.rec_variables.update({rec_variables[i] : np.array(values)})
+            if self.verbose:
+                print 'collected recorded variable %s' % rec_variables[i] 
+            i += 1
+        del self.recvariablesreclist
+    
     def loadspikes(self):
         '''
         Initialize spiketimes from netcon if they exist
@@ -807,12 +808,6 @@ class Cell(object):
                 for ii in xrange(int(self.sptimeslist.o(i).size)):
                     self.netconlist.o(i).event(float(self.sptimeslist.o(i)[ii]))
 
-    def _set_time_recorder(self):
-        '''
-        Initialize time-vector recorder in NEURON
-        '''
-        self.tvec = neuron.h.Vector()
-        self.tvec.record(neuron.h._ref_t, self.timeres_python)
     
     def _set_soma_volt_recorder(self):
         '''
@@ -877,6 +872,31 @@ class Cell(object):
                                               self.timeres_python+1))
                 memvrec.record(seg._ref_v, self.timeres_python)
                 self.memvreclist.append(memvrec)
+
+    
+    def _set_variable_recorders(self, rec_variables):
+        '''
+        Create a recorder for each variable name in list
+        rec_variables
+        
+        Variables is stored in nested list self.recvariablesreclist
+        '''
+        self.recvariablesreclist = neuron.h.List()        
+        for variable in rec_variables:
+            variablereclist = neuron.h.List()
+            self.recvariablesreclist.append(variablereclist)
+            for sec in self.allseclist:
+                for seg in sec:
+                    recvector = neuron.h.Vector(int(self.tstopms /
+                                                    self.timeres_python + 1))
+                    if hasattr(seg, variable):
+                        recvector.record(getattr(seg, '_ref_%s' % variable),
+                                         self.timeres_python)
+                    else:
+                        print 'non-existing variable %s, section %s.%f' % \
+                                (variable, sec.name(), seg.x)
+                    variablereclist.append(recvector)
+        
     
     def set_pos(self, xpos=0, ypos=0, zpos=0):
         '''
@@ -1012,6 +1032,32 @@ class Cell(object):
             if self.verbose:
                 print 'Geometry not rotated around z-axis'
     
+    
+    def chiral_morphology(self, axis='x'):
+        '''
+        Mirror the morphology around given axis, (default x-axis),
+        useful to introduce more heterogeneouties in morphology shapes
+        '''
+        #morphology relative to soma-position
+        rel_start, rel_end = self._rel_positions()
+        if axis == 'x':
+            rel_start[:, 0] = -rel_start[:, 0]
+            rel_end[:, 0] = -rel_end[:, 0]
+        elif axis == 'y':
+            rel_start[:, 1] = -rel_start[:, 1]
+            rel_end[:, 1] = -rel_end[:, 1]
+        elif axis == 'z':
+            rel_start[:, 2] = -rel_start[:, 2]
+            rel_end[:, 2] = -rel_end[:, 2]
+        else:
+            raise Exception, "axis must be either 'x', 'y' or 'z'"
+        
+        if self.verbose:
+            print 'morphology mirrored across %s-axis' % axis
+        
+        #set the proper 3D positions
+        self._real_positions(rel_start, rel_end)
+        
     def _squeeze_me_macaroni(self):
         '''
         Reducing the dimensions of the morphology matrices from 3D->2D
@@ -1185,4 +1231,4 @@ class Cell(object):
                     idxvec[i] += 1
                 i += 1
         [idx] = np.where(idxvec)
-        return idx
\ No newline at end of file
+        return idx
